<?php if (!defined('PNtoMath')) exit ();
/*
 * PNtoMath:
 * Associated Files and Classes
 * Files : 
 *   - PNtoMath.class.php
 *   - PNtoMath.cfg.php
 *
 * PNtoMath Version 0.1, 10 August 2013, (c) Mehmet Durgel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License (at http://www.gnu.org/copyleft/gpl.html)
 * for more details.
 *
 * -- PURPOSE
 * Developed convert ascii formula into HTML or MathML to be displayed in PmWiki 2.x pages
 * This code is an implementation of polish notation (PN) invented by the Polish logician Jan Łukasiewicz.
 * See (http://en.wikipedia.org/wiki/Polish_notation) for details.
 * 
 * -- USAGE NOTES
 * '{','}' : used as word seperators as in example : =/*{height}{base}2{area of triangle}
*/

class PNtoMath{
	protected $stack = array();
	protected $math = array();
	protected $expression = false;
	protected $type = 'MathML'; //HTML also excepted
	protected $arity = 2;

	/**
	 * Main processing function
	 */
	function processAs($type = 'MathML'){
		$this->type = $type;
		$this->produceStack();
		$this->processStack($type);
		return $this->stack[0];
	}

	/**
	 * Sets expression
	 */
	function setExpression($expression){
		$this->expression = html_entity_decode($expression);
	}

	/**
	 * Basic getter function
	 */
	function getMath(){
		return $this->math;
	}

	/**
	 * Reads expression as char sequence and produce a token array
	 */
	function produceStack(){
		$buffer = false;
		$i = 0;
		for($j=0; $j < strlen($this->expression); $j++)
		{
			$char = $this->expression[$j];
			if(in_array($char, array('{','}')))
			{
				if($char == '{')
				{
					$buffer = true;
					$this->stack[$i] = '';
				}
				else if($char == '}')
				{
					$buffer = false;
					$i++;
				}
			}
			else
			{
				if($buffer)
				{
					$this->stack[$i] .= $char;
				}
				else
				{
					$this->stack[$i] = $char;
					$i++;
				}
			}
		}
	}

	/**
	 * Processes stack depending on the type supplied
	*/
	function processStack($type = 'MathML'){
		$operators = getOperators($type);
		$stack = array_reverse($this->stack);

		$math = array();
		for($i = 0; $i < count($stack); $i++)
		{
			$token = $stack[$i];
			if(array_key_exists($token, $operators))
			{
				if(count($math) < $this->arity)
				{
					$this->math = 'Insufficient amount of value tokens!';
				}
				else 
				{
					$method = 'processToken'.$type; 
					$items[0] = $this->$method(trim(array_pop($math)));
					$items[1] = $this->$method(trim(array_pop($math)));
					array_push($math, vsprintf($operators[$token], $items ));
				}
			}else{
				$math[] = $token;
			}
		}

	   	if(count($math) == 1)
		{
			$this->math = $math[0];
		}
		else
		{
			$this->math = 'Too many value tokens supplied!' . '<pre>' . print_r($this->stack, true) . '</pre>';
		}
	}

	/**
	 * Operand wrapper function for HTML output
	 */
	function processTokenHTML($token){
		return "<span>$token</span>";
	}

	/**
	 * Operand wrapper function for MathML output
	 */
	function processTokenMathML($token){
		if(is_numeric($token))
		{
			return "<mn>$token</mn>";
		}
		else
		{
			return "<mi>$token</mi>";
		}
	}
}
